@@ -3,10 +3,13 @@
 # Custom - A class for writing the Excel XLSX Custom Property file.
 #
 # SPDX-License-Identifier: BSD-2-Clause
-# Copyright 2013-2023, John McNamara, jmcnamara@cpan.org
+#
+# Copyright (c) 2013-2025, John McNamara, jmcnamara@cpan.org
 #
 
 # Package imports.
+from typing import List, Tuple
+
 from . import xmlwriter
 
 
@@ -29,12 +32,12 @@ class Custom(xmlwriter.XMLwriter):
 
         """
 
-        super(Custom, self).__init__()
+        super().__init__()
 
         self.properties = []
         self.pid = 1
 
-    def _set_properties(self, properties):
+    def _set_properties(self, properties: List[Tuple[str, str, str]]):
         # Set the document properties.
         self.properties = properties
 
@@ -80,7 +83,7 @@ class Custom(xmlwriter.XMLwriter):
             # Write the property element.
             self._write_property(custom_property)
 
-    def _write_property(self, custom_property):
+    def _write_property(self, custom_property: Tuple[str, str, str]):
         # Write the <property> element.
 
         fmtid = "{D5CDD505-2E9C-101B-9397-08002B2CF9AE}"
@@ -114,28 +117,22 @@ class Custom(xmlwriter.XMLwriter):
 
         self._xml_end_tag("property")
 
-    def _write_vt_lpwstr(self, value):
+    def _write_vt_lpwstr(self, value: str):
         # Write the <vt:lpwstr> element.
         self._xml_data_element("vt:lpwstr", value)
 
-    def _write_vt_filetime(self, value):
+    def _write_vt_filetime(self, value: str):
         # Write the <vt:filetime> element.
         self._xml_data_element("vt:filetime", value)
 
-    def _write_vt_i4(self, value):
+    def _write_vt_i4(self, value: str):
         # Write the <vt:i4> element.
         self._xml_data_element("vt:i4", value)
 
-    def _write_vt_r8(self, value):
+    def _write_vt_r8(self, value: str):
         # Write the <vt:r8> element.
         self._xml_data_element("vt:r8", value)
 
-    def _write_vt_bool(self, value):
+    def _write_vt_bool(self, value: str):
         # Write the <vt:bool> element.
-
-        if value:
-            value = "true"
-        else:
-            value = "false"
-
         self._xml_data_element("vt:bool", value)
