@@ -18,6 +18,8 @@ from fractions import Fraction
 from warnings import warn
 from zipfile import ZIP_DEFLATED, LargeZipFile, ZipFile, ZipInfo
 
+from xlsxwriter.image import Image
+
 # Package imports.
 from . import xmlwriter
 from .chart_area import ChartArea
@@ -39,7 +41,7 @@ from .exceptions import (
 from .format import Format
 from .packager import Packager
 from .sharedstrings import SharedStringTable
-from .utility import _get_image_properties, xl_cell_to_rowcol
+from .utility import xl_cell_to_rowcol
 from .worksheet import Worksheet
 
 
@@ -464,9 +466,16 @@ class Workbook(xmlwriter.XMLwriter):
             else:
                 property_type = "text"
 
+        # Convert non-string values to strings to have a single data type.
+        if property_type == "bool":
+            value = str(value).lower()
+
         if property_type == "date":
             value = value.strftime("%Y-%m-%dT%H:%M:%SZ")
 
+        if property_type in ("number", "number_int"):
+            value = str(value)
+
         if property_type == "text" and len(value) > 255:
             warn(
                 f"Length of 'value' parameter exceeds Excel's limit of 255 "
@@ -956,12 +965,12 @@ class Workbook(xmlwriter.XMLwriter):
             if key in fonts:
                 # Font has already been used.
                 xf_format.font_index = fonts[key]
-                xf_format.has_font = 0
+                xf_format.has_font = False
             else:
                 # This is a new font.
                 fonts[key] = index
                 xf_format.font_index = index
-                xf_format.has_font = 1
+                xf_format.has_font = True
                 index += 1
 
         self.font_count = index
@@ -977,7 +986,7 @@ class Workbook(xmlwriter.XMLwriter):
                 or xf_format.underline
                 or xf_format.font_strikeout
             ):
-                xf_format.has_dxf_font = 1
+                xf_format.has_dxf_font = True
 
     def _prepare_num_formats(self):
         # User defined records in Excel start from index 0xA4.
@@ -1036,24 +1045,24 @@ class Workbook(xmlwriter.XMLwriter):
             if key in borders:
                 # Border has already been used.
                 xf_format.border_index = borders[key]
-                xf_format.has_border = 0
+                xf_format.has_border = False
             else:
                 # This is a new border.
                 borders[key] = index
                 xf_format.border_index = index
-                xf_format.has_border = 1
+                xf_format.has_border = True
                 index += 1
 
         self.border_count = index
 
         # For DXF formats we only need to check if the properties have changed.
-        has_border = re.compile(r"[^0:]")
+        has_border = re.compile(r"[^0None:]")
 
         for xf_format in self.dxf_formats:
             key = xf_format._get_border_key()
 
             if has_border.search(key):
-                xf_format.has_dxf_border = 1
+                xf_format.has_dxf_border = True
 
     def _prepare_fills(self):
         # Iterate through the XF Format objects and give them an index to
@@ -1064,13 +1073,13 @@ class Workbook(xmlwriter.XMLwriter):
         index = 2  # Start from 2. See above.
 
         # Add the default fills.
-        fills["0:0:0"] = 0
-        fills["17:0:0"] = 1
+        fills["0:None:None"] = 0
+        fills["17:None:None"] = 1
 
         # Store the DXF colors separately since them may be reversed below.
         for xf_format in self.dxf_formats:
             if xf_format.pattern or xf_format.bg_color or xf_format.fg_color:
-                xf_format.has_dxf_fill = 1
+                xf_format.has_dxf_fill = True
                 xf_format.dxf_bg_color = xf_format.bg_color
                 xf_format.dxf_fg_color = xf_format.fg_color
 
@@ -1082,29 +1091,17 @@ class Workbook(xmlwriter.XMLwriter):
             # 2. If the user specifies a foreground or background color
             # without a pattern they probably wanted a solid fill, so we fill
             # in the defaults.
-            if (
-                xf_format.pattern == 1
-                and xf_format.bg_color != 0
-                and xf_format.fg_color != 0
-            ):
+            if xf_format.pattern == 1 and xf_format.bg_color and xf_format.fg_color:
                 tmp = xf_format.fg_color
                 xf_format.fg_color = xf_format.bg_color
                 xf_format.bg_color = tmp
 
-            if (
-                xf_format.pattern <= 1
-                and xf_format.bg_color != 0
-                and xf_format.fg_color == 0
-            ):
+            if xf_format.pattern <= 1 and xf_format.bg_color and not xf_format.fg_color:
                 xf_format.fg_color = xf_format.bg_color
-                xf_format.bg_color = 0
+                xf_format.bg_color = None
                 xf_format.pattern = 1
 
-            if (
-                xf_format.pattern <= 1
-                and xf_format.bg_color == 0
-                and xf_format.fg_color != 0
-            ):
+            if xf_format.pattern <= 1 and not xf_format.bg_color and xf_format.fg_color:
                 xf_format.pattern = 1
 
             key = xf_format._get_fill_key()
@@ -1112,12 +1109,12 @@ class Workbook(xmlwriter.XMLwriter):
             if key in fills:
                 # Fill has already been used.
                 xf_format.fill_index = fills[key]
-                xf_format.has_fill = 0
+                xf_format.has_fill = False
             else:
                 # This is a new fill.
                 fills[key] = index
                 xf_format.fill_index = index
-                xf_format.has_fill = 1
+                xf_format.has_fill = True
                 index += 1
 
         self.fill_count = index
@@ -1214,10 +1211,11 @@ class Workbook(xmlwriter.XMLwriter):
         background_ids = {}
 
         # Store the image types for any embedded images.
-        for image_data in self.embedded_images.images:
-            image_type = image_data[1]
-            self.image_types[image_type] = True
-            if image_data[3]:
+        for image in self.embedded_images.images:
+            image_extension = image._image_extension
+            self.image_types[image_extension] = True
+
+            if image.description is not None:
                 self.has_embedded_descriptions = True
 
         image_ref_id = len(self.embedded_images.images)
@@ -1249,70 +1247,42 @@ class Workbook(xmlwriter.XMLwriter):
 
             # Prepare the background images.
             if sheet.background_image:
-                if sheet.background_bytes:
-                    filename = ""
-                    image_data = sheet.background_image
-                else:
-                    filename = sheet.background_image
-                    image_data = None
+                image = sheet.background_image
+                image_extension = image._image_extension
+                image_digest = image._digest
 
-                (
-                    image_type,
-                    _,
-                    _,
-                    _,
-                    _,
-                    _,
-                    digest,
-                ) = _get_image_properties(filename, image_data)
+                self.image_types[image_extension] = True
 
-                self.image_types[image_type] = True
-
-                if digest in background_ids:
-                    ref_id = background_ids[digest]
+                if image_digest in background_ids:
+                    ref_id = background_ids[image_digest]
                 else:
                     image_ref_id += 1
                     ref_id = image_ref_id
-                    background_ids[digest] = image_ref_id
-                    self.images.append([filename, image_type, image_data])
+                    background_ids[image_digest] = image_ref_id
+                    self.images.append(image)
 
-                sheet._prepare_background(ref_id, image_type)
+                sheet._prepare_background(ref_id, image_extension)
 
             # Prepare the worksheet images.
             for index in range(image_count):
-                filename = sheet.images[index][2]
-                image_data = sheet.images[index][10]
-                (
-                    image_type,
-                    width,
-                    height,
-                    name,
-                    x_dpi,
-                    y_dpi,
-                    digest,
-                ) = _get_image_properties(filename, image_data)
+                image = sheet.images[index]
+                image_extension = image._image_extension
+                image_digest = image._digest
 
-                self.image_types[image_type] = True
+                self.image_types[image_extension] = True
 
-                if digest in image_ids:
-                    ref_id = image_ids[digest]
+                if image_digest in image_ids:
+                    ref_id = image_ids[image_digest]
                 else:
                     image_ref_id += 1
                     ref_id = image_ref_id
-                    image_ids[digest] = image_ref_id
-                    self.images.append([filename, image_type, image_data])
+                    image_ids[image_digest] = image_ref_id
+                    self.images.append(image)
 
                 sheet._prepare_image(
-                    index,
+                    image,
                     ref_id,
                     drawing_id,
-                    width,
-                    height,
-                    name,
-                    image_type,
-                    x_dpi,
-                    y_dpi,
-                    digest,
                 )
 
             # Prepare the worksheet charts.
@@ -1325,80 +1295,38 @@ class Workbook(xmlwriter.XMLwriter):
                 sheet._prepare_shape(index, drawing_id)
 
             # Prepare the header images.
-            for index in range(header_image_count):
-                filename = sheet.header_images[index][0]
-                image_data = sheet.header_images[index][1]
-                position = sheet.header_images[index][2]
+            for image in sheet.header_images:
+                image_extension = image._image_extension
+                image_digest = image._digest
 
-                (
-                    image_type,
-                    width,
-                    height,
-                    name,
-                    x_dpi,
-                    y_dpi,
-                    digest,
-                ) = _get_image_properties(filename, image_data)
+                self.image_types[image_extension] = True
 
-                self.image_types[image_type] = True
-
-                if digest in header_image_ids:
-                    ref_id = header_image_ids[digest]
+                if image_digest in header_image_ids:
+                    ref_id = header_image_ids[image_digest]
                 else:
                     image_ref_id += 1
                     ref_id = image_ref_id
-                    header_image_ids[digest] = image_ref_id
-                    self.images.append([filename, image_type, image_data])
+                    header_image_ids[image_digest] = image_ref_id
+                    self.images.append(image)
 
-                sheet._prepare_header_image(
-                    ref_id,
-                    width,
-                    height,
-                    name,
-                    image_type,
-                    position,
-                    x_dpi,
-                    y_dpi,
-                    digest,
-                )
+                sheet._prepare_header_image(ref_id, image)
 
             # Prepare the footer images.
-            for index in range(footer_image_count):
-                filename = sheet.footer_images[index][0]
-                image_data = sheet.footer_images[index][1]
-                position = sheet.footer_images[index][2]
+            for image in sheet.footer_images:
+                image_extension = image._image_extension
+                image_digest = image._digest
 
-                (
-                    image_type,
-                    width,
-                    height,
-                    name,
-                    x_dpi,
-                    y_dpi,
-                    digest,
-                ) = _get_image_properties(filename, image_data)
+                self.image_types[image_extension] = True
 
-                self.image_types[image_type] = True
-
-                if digest in header_image_ids:
-                    ref_id = header_image_ids[digest]
+                if image_digest in header_image_ids:
+                    ref_id = header_image_ids[image_digest]
                 else:
                     image_ref_id += 1
                     ref_id = image_ref_id
-                    header_image_ids[digest] = image_ref_id
-                    self.images.append([filename, image_type, image_data])
+                    header_image_ids[image_digest] = image_ref_id
+                    self.images.append(image)
 
-                sheet._prepare_header_image(
-                    ref_id,
-                    width,
-                    height,
-                    name,
-                    image_type,
-                    position,
-                    x_dpi,
-                    y_dpi,
-                    digest,
-                )
+                sheet._prepare_header_image(ref_id, image)
 
             if has_drawing:
                 drawing = sheet.drawing
@@ -1821,24 +1749,23 @@ class EmbeddedImages:
         self.images = []
         self.image_indexes = {}
 
-    def get_image_index(self, image, digest):
+    def get_image_index(self, image: Image):
         """
         Get the index of an embedded image.
 
         Args:
             image: The image to lookup.
-            digest: The digest of the image.
 
         Returns:
             The image index.
 
         """
-        image_index = self.image_indexes.get(digest)
+        image_index = self.image_indexes.get(image._digest)
 
         if image_index is None:
             self.images.append(image)
             image_index = len(self.images)
-            self.image_indexes[digest] = image_index
+            self.image_indexes[image._digest] = image_index
 
         return image_index
 
